@@ -6,7 +6,7 @@
    //
    //    Part of the ht://Check package
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Some Portions Copyright (c) 1995-2000 The ht://Dig Group <www.htdig.org>
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
@@ -14,19 +14,34 @@
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: Scheduler.cc,v 1.47 2002/02/18 14:21:50 mnencia Exp $
+   //    $Id: Scheduler.cc,v 1.82 2004/04/06 08:21:25 angusgb Exp $
    //
    //    G.Bartolini
    //    started: 13.09.1999
 ///////
 
-#include "_Server.h"
-#include "Scheduler.h"
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
+#ifdef HAVE_STD
+#include <iostream>
+#include <iomanip>
+#ifdef HAVE_NAMESPACES
+using namespace std;
+#endif
+#else
 #include <iostream.h>
 #include <iomanip.h>
+#endif /* HAVE_STD */
+
+#include "_Server.h"
+#include "Scheduler.h"
+#include "WordType.h"
 #include "HtmlParser.h"
 #include "Htmysql.h"
 #include "HtCookieMemJar.h"
+#include "HtCookieInFileJar.h"
 
 ///////
    //    Construction
@@ -39,12 +54,14 @@ CurrentServer(0),
 CurrentUrl(0),
 Proxy(0),
 Credentials(0),
+ProxyCredentials(0),
 AcceptLanguage(0),
 TransportConnect(0),
 HTTPConnect(0),
 CurrentResponse(0),
 erase(0),
 stop(0), 
+drop_database(true), 
 deserialized(false),
 deserialized_cookies(false),
 _cookie_jar(0),
@@ -167,13 +184,6 @@ int Scheduler::Initial(const String &list)
       }
    }
 
-   // Set the debug level for the other classes
-   Transport::SetDebugLevel(debug);   
-   HtCookieJar::SetDebugLevel(debug);
-   
-   // Set the default parser content-type string
-   Transport::SetDefaultParserContentType ("text/html");
-
    return 1;
 }
 
@@ -283,6 +293,30 @@ Scheduler::Scheduler_Codes Scheduler::SelectDatabase(const String &db_name)
       }
 
    }
+   else
+   {
+      // We have to erase the database. However, let's see if we have
+      // to keep the structure. In order to do this, we have to check
+      // if the database exists.
+
+      if (! drop_database)
+      {
+         if (debug > 0)
+            cout << "Looking if database '" << DB->GetDBSignature() 
+               << "' exists ... should we keep it? ";      
+   
+         if (! DB->Exists(DB->GetDB()) )
+         {
+            // We are looking for an existing database called 'db_name'
+
+            if (debug > 0)
+               cout << "sorry, not found (can't keep it)" << endl;      
+
+            drop_database = true; // We gotta erase the database
+         }
+         else cout << "yes" << endl;
+      }
+   }
 
 ///////
    //   Database creation
@@ -305,6 +339,12 @@ Scheduler::Scheduler_Codes Scheduler::SelectDatabase(const String &db_name)
          cout << endl;
       }
 
+      // Set the available charsets list
+      DB->LoadAvailableCharsets((*Config)["available_charsets"]);
+
+      // Set the 'drop_database' flag
+      DB->SetDropDatabase(drop_database);
+
       if (!DB->CreateDatabase ())
          return Scheduler_DBError;
    
@@ -366,10 +406,14 @@ Scheduler::Scheduler_Codes Scheduler::AddUrl(const String &u, unsigned int &IDUr
    int NumRecords;
    static HtmysqlQueryResult scheduletmp;
 
+   // It's a new Url ... Let's store it
+   _Url tmp(u); // temporary _Url object
+   tmp.normalize();  // Url normalized
+
    // CurrentLinkSchedule Assignment (it contains the current link examinated
    // by the Scheduler) and it's used by parsing functions like HtmlParser's
-   CurrentLinkSchedule.SetNewUrl(u);
-   
+   CurrentLinkSchedule.SetNewUrl(tmp.get());
+
    // Check if the occurrence is already present in the DB
    // NumRecords stores the result of the search process
    // If -1 an error has occurred, else it contains the number of records found
@@ -402,10 +446,6 @@ Scheduler::Scheduler_Codes Scheduler::AddUrl(const String &u, unsigned int &IDUr
       
    }
 
-   // It's a new Url ... Let's store it
-
-   _Url tmp(CurrentLinkSchedule.GetScheduleUrl()); // temporary _Url object
-   tmp.normalize();  // Url normalized
 
    _Server *server = 0;
    
@@ -465,6 +505,7 @@ Scheduler::Scheduler_Codes Scheduler::AddUrl(const String &u, unsigned int &IDUr
       // anyway, kinda force it, right?
 
       CurrentLinkSchedule.SetStatus(SchedulerEntry::Url_ToBeRetrieved);   
+      CurrentLinkSchedule.SetDomain(SchedulerEntry::Url_Internal);
 
    }
    else
@@ -483,6 +524,7 @@ Scheduler::Scheduler_Codes Scheduler::AddUrl(const String &u, unsigned int &IDUr
       case Scheduler_URL_Valid:
          // Valid URL - It has to be retrieved
          CurrentLinkSchedule.SetStatus(SchedulerEntry::Url_ToBeRetrieved);   
+         CurrentLinkSchedule.SetDomain(SchedulerEntry::Url_Internal);
          ++runinfo.TotUrls;
          break;
 
@@ -531,6 +573,7 @@ Scheduler::Scheduler_Codes Scheduler::AddUrl(const String &u, unsigned int &IDUr
          if (debug > 2)
             cout << "    > Rejected: according to the limits list" << endl;
          CurrentLinkSchedule.SetStatus(SchedulerEntry::Url_CheckIfExists);   
+         CurrentLinkSchedule.SetDomain(SchedulerEntry::Url_External);   
          if (Config->Boolean("check_external")) ++runinfo.TotUrls;
          break;
 
@@ -655,6 +698,21 @@ Scheduler::Scheduler_Codes Scheduler::DeserializeCookies()
 void Scheduler::SetOptions(Configuration &config)
 {
 
+   // Set the URL class static configuration variable
+   // IMPORTANT! This must be set before any deal with a URL object
+   URL::SetConfiguration(config);
+      
+   // Set the URL class static configuration variable
+   WordType::Initialize(config);
+
+   // Set the debug level for the other classes
+   Transport::SetDebugLevel(debug);   
+   HtCookieJar::SetDebugLevel(debug);
+   
+   // Set the default parser content-type string
+   Transport::SetDefaultParserContentType ("text/html");
+   
+   // Temporary variables
    StringList l;
    String t;
    String lowerp;
@@ -726,6 +784,9 @@ void Scheduler::SetOptions(Configuration &config)
    // Set the credentials for the authentication
    Credentials = config["authorization"];
 
+   // Set the credentials for the authentication of the HTTP Proxy
+   ProxyCredentials = config["http_proxy_authorization"];
+
    // Set the Accept-Language directive to be sent via HTTP
    l.Create(config["accept_language"], " \t");
    AcceptLanguage.trunc(); // zeroes the contents (should be already empty)
@@ -742,7 +803,34 @@ void Scheduler::SetOptions(Configuration &config)
       cout << "   Setting language for negotiation: "
       	 << ((AcceptLanguage.length()>0)?
 	     AcceptLanguage: String("servers default")) << endl;
-   
+
+   if (!config.Boolean("disable_cookies"))
+   {
+       // Imports the cookies file
+       const String CookiesInputFile = config["cookies_input_file"];
+       if (CookiesInputFile.length())
+       {
+          if (debug>0)
+             cout << "   Importing Cookies input file "
+                 << CookiesInputFile << endl;
+
+	   int result;
+	    if (HtCookieInFileJar* cookie_file = (new HtCookieInFileJar(CookiesInputFile, result)))
+	    {
+		if (!result)
+		{
+		    if (debug>0)
+			cookie_file->ShowSummary();
+		    delete _cookie_jar;	// Deletes previous cookie jar
+		    _cookie_jar = (HtCookieJar*) cookie_file; // set the imported one
+		    HtHTTP::SetCookieJar(_cookie_jar);
+		}
+		else
+		    cout << "! Import failed: " << CookiesInputFile << endl;
+	    }
+	}
+    }
+
    // Set the COnfiguration pointer
    Config = &config;
 
@@ -871,7 +959,7 @@ Scheduler::Scheduler_Codes Scheduler::Run()
       CurrentUrl = new _Url(CurrentSchedule.GetScheduleUrl());
       CurrentUrl->SetID(CurrentSchedule.GetIDSchedule());
       CurrentUrl->SetIDServer(CurrentSchedule.GetIDServer());
-//      CurrentUrl->normalize();
+      // CurrentUrl->normalize();
 
       // Shows info
       if (debug>1)
@@ -919,7 +1007,7 @@ Scheduler::Scheduler_Codes Scheduler::Run()
 
       	    // HTTP response specific matters
       	    if (HTTPConnect == TransportConnect)
-	    {
+	        {
                // Transfer Encoding
                CurrentUrl->SetTransferEncoding(
                   ((HtHTTP_Response *)CurrentResponse)->GetTransferEncoding());
@@ -927,11 +1015,12 @@ Scheduler::Scheduler_Codes Scheduler::Run()
                // Content Language
                CurrentUrl->SetContentLanguage(
                   ((HtHTTP_Response *)CurrentResponse)->GetContentLanguage());
-	    }
+	        }
 
             // We store the server info if it's the first request
             if (CurrentServer->GetRequests() == 0)
             {
+			   CurrentServer->SetIPAddress(TransportConnect->GetHostIPAddress());
                if (HTTPConnect == TransportConnect)
                {
                   // HTTP Info
@@ -976,19 +1065,28 @@ Scheduler::Scheduler_Codes Scheduler::Run()
             if(debug>2)
                cout << "   > Document found" << endl;
 
-            CurrentUrl->SetConnStatus("OK");
-            switch(HtmlParserOB(*this))
-	    {
-	       case HtmlParser::HtmlParser_StatementFailed:
-	       case HtmlParser::HtmlParser_LinkFailed:
-	       case HtmlParser::HtmlParser_AttributeFailed:
-      	          return Scheduler_DBError;
-	          break;
-		  
-	       default:
-	          break;
-	    }
-	    break;
+			CurrentUrl->SetConnStatus("OK");
+			if (CurrentSchedule.GetStatus()
+				!= SchedulerEntry::Url_CheckIfExists)
+			{
+				switch(HtmlParserOB(*this))
+				{
+					case HtmlParser::HtmlParser_StatementFailed:
+					case HtmlParser::HtmlParser_LinkFailed:
+					case HtmlParser::HtmlParser_AttributeFailed:
+					case HtmlParser::HtmlParser_AccessibilityCheckFailed:
+						return Scheduler_DBError;
+						break;
+		  			
+					default:
+						if (Config->Boolean("store_url_contents"))
+							CurrentUrl->SetContents(
+								(const String* const)
+								(&(CurrentResponse->GetContents())));
+						break;
+				}
+			}
+			break;
 
 
          ///////
@@ -1035,6 +1133,7 @@ Scheduler::Scheduler_Codes Scheduler::Run()
                static Link redirectedlink;
 
                CurrentUrl->SetLocation(((HtHTTP_Response *)CurrentResponse)->GetLocation());
+               CurrentUrl->SetContentType(((HtHTTP_Response *)CurrentResponse)->GetContentType());
 
                _Url RedirectedUrl (CurrentUrl->GetLocation(),
                   *CurrentUrl);
@@ -1132,6 +1231,12 @@ Scheduler::Scheduler_Codes Scheduler::Run()
             CurrentUrl->SetConnStatus("ServiceNotValid");
             break;
             
+         case Transport::Document_server_error:
+            if(debug>0)
+               cout << "   > Server Error" << endl;
+            CurrentUrl->SetConnStatus("ServerError");
+            break;
+            
          case Transport::Document_other_error:
             // General error (memory)
             return Scheduler_MemoryError;
@@ -1179,7 +1284,7 @@ Scheduler::Scheduler_Codes Scheduler::Run()
       return Scheduler_DBError;
 
    // Set the link results
-   if (SetLinkResults() != Scheduler_OK)
+   if (DB->SetLinkResults() == -1)
       return Scheduler_DBError;
 
    // Set the HTML anchors results
@@ -1212,7 +1317,11 @@ Scheduler::Scheduler_Codes Scheduler::Run()
    runinfo.HTTPSeconds= HtHTTP::GetTotSeconds();
    runinfo.HTTPRequests= HtHTTP::GetTotRequests();
    runinfo.HTTPBytes= HtHTTP::GetTotBytes();
+   runinfo.HTTPBytes= HtHTTP::GetTotBytes();
 
+   // Updates accessibility checks info
+   if (!Config->Boolean("accessibility_checks"))
+      runinfo.AccessibilityChecks = 0;
    
    // Store general info into the'htCheck' table
    if (! DB->Insert(runinfo))
@@ -1363,6 +1472,15 @@ Transport::DocStatus Scheduler::Retrieve (const SchedulerEntry &s, _Url &url)
 
          HTTPConnect->SetProxy(useproxy); // Set the flag for HTTP proxy
          TransportConnect->SetConnection(Proxy);
+
+         // Set the credentials for the authentication
+         if (ProxyCredentials.length())
+	 {
+            if (debug>3)
+               cout << "Set the authorization for the proxy" << endl;
+
+            HTTPConnect->SetProxyCredentials(ProxyCredentials);
+	 }
          
       }
       else TransportConnect->SetConnection(url);
@@ -1461,6 +1579,7 @@ int Scheduler::GetNext(const String &StrStatus)
 {
 
    int NumRecords = 0;
+   const String SQLCommonStatement = "Select IDUrl, IDServer, Url, Status, IDReferer, HopCount from Schedule";
 
    if (!ScheduleTmp.Empty() && ScheduleTmp.Type() == Htmysql::Htmysql_Stored)
    {
@@ -1473,7 +1592,7 @@ int Scheduler::GetNext(const String &StrStatus)
       if (DB->GetNextElement(CurrentSchedule, ScheduleTmp)) // found an item
       {
          if (debug>0)
-            cout << "-";
+            cout << "- ";
 
          return 1;
       }
@@ -1491,12 +1610,10 @@ int Scheduler::GetNext(const String &StrStatus)
       // These Urls won't increase the list of Urls to be retrieved
       // So we query the DB only once. Example: 'CheckIfExists'
          
-         
       if (debug>2)
          cout << "Creating a new list of Urls of type: " << StrStatus << endl;
 
-      String SQLStatement="Select * from Schedule";
-         
+      String SQLStatement = SQLCommonStatement;
       SQLStatement << " where Status='" << StrStatus << "'";
       SQLStatement << " ORDER by IDServer, HopCount ASC";
 
@@ -1511,12 +1628,12 @@ int Scheduler::GetNext(const String &StrStatus)
          // We found at least one record
 
          if (debug>0)
-            cout << "= " << NumRecords << " Urls." << endl;
+            cout << "+ " << NumRecords << " Urls." << endl;
 
          if (DB->GetNextElement(CurrentSchedule, ScheduleTmp) )
          {
             if (debug>0)
-               cout << "-";
+               cout << "- ";
             return 1;
          }
       }
@@ -1531,8 +1648,7 @@ int Scheduler::GetNext(const String &StrStatus)
                << CurrentServer->host() << ":" << CurrentServer->port()
                << " - persistent connections - type: " << StrStatus << endl;
 
-         String SQLStatement="Select * from Schedule";
-         
+         String SQLStatement = SQLCommonStatement;
          SQLStatement << " where Status='" << StrStatus << "'";
          SQLStatement << " AND IDServer = " << CurrentServer->GetID()
             << " ORDER BY HopCount ASC";
@@ -1554,7 +1670,11 @@ int Scheduler::GetNext(const String &StrStatus)
             // We found at least one record
 
             if (DB->GetNextElement(CurrentSchedule, ScheduleTmp))
-            return 1;
+            {
+               if (debug>0)
+                  cout << "- ";
+               return 1;
+            }
             
          }
 
@@ -1563,8 +1683,7 @@ int Scheduler::GetNext(const String &StrStatus)
       
    // We take the first records, as is with a temporary query
 
-   String SQLStatement="Select * from Schedule";
-         
+   String SQLStatement = SQLCommonStatement;
    SQLStatement << " where Status='" << StrStatus << "'"
             << " ORDER BY HopCount ASC";
 
@@ -1577,7 +1696,7 @@ int Scheduler::GetNext(const String &StrStatus)
    NumRecords=DB->GetNextElement(CurrentSchedule, ScheduleTmp);
       
    if (NumRecords & debug>0)
-      cout << "?";
+      cout << "+ ";
          
    ScheduleTmp.Free();
 
@@ -1694,12 +1813,13 @@ Scheduler::Scheduler_Codes Scheduler::ShowBrokenLinks(ostream &output)
       << "Link.IDUrlDest as IDUrlDest, "
       << "Url.Url as UrlSrc, "
       << "Schedule.Url as UrlDest, "
-      << "HtmlStatement.Statement "
+      << "HtmlStatement.Statement, "
+      << "Link.LinkResult as LinkResult "
       << "FROM Url, Schedule, Link "
       << "LEFT JOIN HtmlStatement "
       << "ON HtmlStatement.IDUrl = Link.IDUrlSrc "
       << "AND HtmlStatement.TagPosition = Link.TagPosition "
-      << "WHERE Link.LinkResult = 'Broken' "
+      << "WHERE (Link.LinkResult = 'Broken' OR Link.LinkResult = 'BadEncoded')"
       << "AND Schedule.IDUrl = Link.IDUrlDest "
       << "AND Url.IDUrl = Link.IDUrlSrc "
       << "ORDER BY UrlSrc, UrlDest, Link.TagPosition, "
@@ -1731,7 +1851,12 @@ Scheduler::Scheduler_Codes Scheduler::ShowBrokenLinks(ostream &output)
          // New URL not found
          ++NumUrlsNotFound;
          output << endl;
-         output << "URL Not found: " << (char *) BrokenLinks[3] << endl;
+         if (!strncmp("BadEncoded", BrokenLinks[5], 10))
+            output << "Bad encoded link: ";
+         else
+            output << "URL Not found: ";
+
+         output << (char *) BrokenLinks[3] << endl;
 
          OldIDDest = atoi(BrokenLinks[1]); // New assignment
          OldIDSrc = 0;
@@ -1855,8 +1980,12 @@ Scheduler::Scheduler_Codes Scheduler::ShowContentTypesPerServer(ostream &output)
    HtmysqlQueryResult ResultTmp;
    
    int NumServer = 0;
+   int TotNumTot = 0;
+   int TotNumImg = 0;
+   int TotNumHTML = 0;
    int NumTot = 0;
    int NumImg = 0;
+   int NumHTML = 0;
    String OldServer = 0;
    String NewServer = 0;
    int OldPort = 0;
@@ -1885,8 +2014,15 @@ Scheduler::Scheduler_Codes Scheduler::ShowContentTypesPerServer(ostream &output)
             // Close a previous server
             
             output << "      Total Urls successfully seen: " << NumTot
-               << " (images: " << NumImg << " - other: " << NumTot-NumImg
+               << " (HTML: " << NumHTML
+               << " - images: " << NumImg
+			   << " - other: " << NumTot-NumImg-NumHTML
                << ")" << endl;
+
+			// Updates the total values
+			TotNumTot += NumTot;
+			TotNumImg += NumImg;
+			TotNumHTML += NumHTML;
             
          }
 
@@ -1896,6 +2032,7 @@ Scheduler::Scheduler_Codes Scheduler::ShowContentTypesPerServer(ostream &output)
          
          NumTot = 0;
          NumImg = 0;
+         NumHTML = 0;
          
          OldServer = NewServer;
          OldPort = NewPort;
@@ -1912,6 +2049,9 @@ Scheduler::Scheduler_Codes Scheduler::ShowContentTypesPerServer(ostream &output)
       if (!strncmp("image/", (char *)StatusCodes[2], 6))
          NumImg += atoi(StatusCodes[3]);
 
+      if (!strncmp("text/html", (char *)StatusCodes[2], 9))
+         NumHTML += atoi(StatusCodes[3]);
+
    }
 
    ResultTmp.Free();    // Free the query results
@@ -1921,14 +2061,26 @@ Scheduler::Scheduler_Codes Scheduler::ShowContentTypesPerServer(ostream &output)
       // Close a previous server
             
       output << "      Total Urls successfully seen: " << NumTot
-         << " (images: " << NumImg << " - other: " << NumTot-NumImg
+         << " (HTML: " << NumHTML
+         << " - images: " << NumImg
+		 << " - other: " << NumTot-NumImg-NumHTML
          << ")" << endl;
             
+      // Updates the total values
+      TotNumTot += NumTot;
+      TotNumImg += NumImg;
+      TotNumHTML += NumHTML;
+            
    }
 
    // Flush an empty line ans display the number of servers that have been
-   // crawled by htcheck
-   output << endl << " Servers seen: " << NumServer << endl << endl;
+   // crawled by htcheck and info regarding the types globally
+   output << endl << " Total Urls successfully seen: " << TotNumTot
+      << " (HTML: " << TotNumHTML
+      << " - images: " << TotNumImg
+	  << " - other: " << TotNumTot-TotNumImg-TotNumHTML
+      << ")" << endl;
+   output << " Servers seen: " << NumServer << endl << endl;
 
    return Scheduler_OK;
    
@@ -1958,6 +2110,7 @@ Scheduler::Scheduler_Codes Scheduler::CalculateUrlSizeAdd()
    String SQLStatement="Select distinct Link.IDUrlSrc, Link.IDUrlDest, Url.Size";
    SQLStatement << " from Link, Url"
       << " where Link.LinkType='Direct' AND Link.IDUrlDest=Url.IDUrl"
+      << " AND Url.Size > 0"
       << " Order by Link.IDUrlSrc";
    
    HtmysqlQueryResult ResultTmp;
@@ -2049,108 +2202,6 @@ Scheduler::Scheduler_Codes Scheduler::CalculateUrlSizeAdd()
 }
 
 
-///////
-   //    Look for all the links that have 'NotChecked' flag in the LinkResult
-   //    field. For each of them, it checks all of the links, and see if
-   //    they are eithrt found or not retrieved or broken or redirected.
-   //    Then it updates the Link table with the right flag.
-///////
-
-Scheduler::Scheduler_Codes Scheduler::SetLinkResults(ostream &output)
-{
-
-   // Set the link results
-
-   String SQLStatement=0;
-   Link LinkTmp;  // Temporary link object
-   
-   SQLStatement << "SELECT distinct IDUrlSrc, IDUrlDest, "
-      << "Url.StatusCode "
-      << "FROM Link LEFT JOIN Url on Link.IDUrlDest=Url.IDUrl "
-      << "WHERE Link.LinkResult='NotChecked'";
-
-   HtmysqlQueryResult ResultTmp;
-   
-   int NumLinks = 0;
-
-   if (debug>0)
-      output << endl << "Setting link results" << endl;
-
-   if (DB->Query (SQLStatement, &ResultTmp) == -1)
-      return Scheduler_DBError;
-
-   String strResult;
-   MYSQL_ROW LinkRow;
-   
-   while ((LinkRow = (ResultTmp.GetNextRecord())))
-   {
-      unsigned int StatusCode;
-      
-      ++NumLinks;
-      
-      LinkTmp.Reset();  
-      LinkTmp.SetIDUrlSrc((unsigned int) atoi(LinkRow[0]));
-      LinkTmp.SetIDUrlDest((unsigned int) atoi(LinkRow[1]));
-
-      if (LinkRow[2])
-      {
-         StatusCode=(unsigned int) atoi(LinkRow[2]);
-         if (StatusCode == 200)
-         {
-            // Found
-            LinkTmp.SetLinkResult(Link::Link_OK);
-         
-         if (debug>2)
-               output << "Link found: " << LinkTmp << endl;
-         }
-         else if (StatusCode >= 300 && StatusCode < 400)
-         {
-            // Redirected
-            LinkTmp.SetLinkResult(Link::Link_Redirected);
-         
-         if (debug>2)
-               output << "Link redirected: " << LinkTmp << endl;
-         }
-         else if (StatusCode == 401)
-         {
-            // Not autorized
-            LinkTmp.SetLinkResult(Link::Link_NotAuthorized);
-         
-         if (debug>2)
-               output << "Link not authorized: " << LinkTmp << endl;
-         }
-         else
-         {
-            // Broken
-            LinkTmp.SetLinkResult(Link::Link_Broken);
-         
-         if (debug>2)
-               output << "Link broken: " << LinkTmp << endl;
-         }
-      }
-      else LinkTmp.SetLinkResult(Link::Link_NotRetrieved); // Not Retrieved
-      
-      LinkTmp.RetrieveLinkResult(strResult);
-
-      String SQLUpdateStatement=0;
-      SQLUpdateStatement << "Update Link SET LinkResult = '" << strResult << "'"
-         << " WHERE IDUrlSrc = " << LinkTmp.GetIDUrlSrc()
-         << " AND IDUrlDest = " << LinkTmp.GetIDUrlDest();
-
-      if(DB->Query(SQLUpdateStatement) == -1) // A database error occured
-         return Scheduler_DBError;
-
-   }
-
-   ResultTmp.Free();      
-   if (debug>0)
-      output << "Links checked: " << NumLinks << endl;
-
-   return Scheduler_OK;
-   
-}
-
-
 ///////
    //    Check the HTML anchors
 ///////
@@ -2158,93 +2209,14 @@ Scheduler::Scheduler_Codes Scheduler::SetLinkResults(ostream &output)
 Scheduler::Scheduler_Codes Scheduler::SetHTMLAnchorsResults(ostream &output)
 {
 
-   // Set the link results
-
-   String SQLStatement=0;
-   Link LinkTmp;  // Temporary link object
-
    if (debug>0)
       output << endl << "Setting HTML Anchors results" << endl;
 
    // Create the table with all the anchors;
 
-   if(DB->CreateAnchorsTable() == -1) // A database error occured
+   if(DB->AnchorsTable(output) == -1) // A database error occured
       return Scheduler_DBError;
 
-   // Now the temporary table is filled with all the anchors found
-
-   SQLStatement << "SELECT DISTINCT Link.IDUrlSrc, Link.IDUrlDest, "
-      << "Link.TagPosition, Link.AttrPosition, Link.Anchor, TmpAnchors.Anchor "
-      << "FROM Link, Schedule LEFT JOIN TmpAnchors ON "
-      << "TmpAnchors.IDUrl=Link.IDUrlDest "
-      << "AND TmpAnchors.Anchor=Link.Anchor "
-      << "WHERE Link.Anchor != '' AND LCASE(Link.Anchor) != 'top' "
-      << "AND Link.LinkResult='OK' "
-      << "AND Link.IDUrlDest=Schedule.IDUrl AND Schedule.Status='Retrieved'";
-  
-   HtmysqlQueryResult ResultTmp;
-   
-   int NumAnchors = 0;
-   int NumAnchorsNotFound = 0;
-
-   if (DB->Query (SQLStatement, &ResultTmp) == -1)
-      return Scheduler_DBError;
-
-   MYSQL_ROW LinkRow;
-   
-   while ((LinkRow = (ResultTmp.GetNextRecord())))
-   {
-      ++NumAnchors;
-      
-      if (! LinkRow[5])
-      {
-         // Anchor Not found
-         
-         ++NumAnchorsNotFound;
-         LinkTmp.Reset();  
-         LinkTmp.SetIDUrlSrc((unsigned int) atoi(LinkRow[0]));
-         LinkTmp.SetIDUrlDest((unsigned int) atoi(LinkRow[1]));
-         LinkTmp.SetTagPosition((unsigned int) atoi(LinkRow[2]));
-         LinkTmp.SetAttrPosition((unsigned int) atoi(LinkRow[3]));
-         LinkTmp.SetAnchor(LinkRow[4]);
-
-         LinkTmp.SetLinkResult(Link::Link_AnchorNotFound);
-         
-         if (debug>2)
-               output << "Anchor not found: " << LinkTmp
-                  << "#" << LinkTmp.GetAnchor() << endl;
-
-         // Update the link table - setting the results
-
-         if(DB->SetAnchorsResults(LinkTmp) == -1) // A database error occured
-            return Scheduler_DBError;
-      }
-
-   }
-
-   ResultTmp.Free();      
-
-   if (debug>0)
-      output << "Anchors checked: " << NumAnchors
-         << " - Not found: " << NumAnchorsNotFound << endl;
-
-
-///////
-   //    Drop the 'TmpAnchors' temporary table
-///////
-
-   if (debug >1)
-      output << " |- Dropping 'TmpAnchors' temporary table" << endl;
-
-   // Write the SQL statement
-   SQLStatement.trunc();
-   SQLStatement << "DROP TABLE " << DB->GetDB() << ".TmpAnchors";
-
-   // Executing table dropping
-
-   if (DB->Query (SQLStatement) == -1)
-      return Scheduler_DBError;  // An error occured
-
    return Scheduler_OK;
    
 }
@@ -2264,9 +2236,9 @@ bool Scheduler::UseProxy(const SchedulerEntry &s)
    url = s.GetScheduleUrl();
 
    if (ExcludeProxy.match(url.get(), 0, 0) == 0)
-      return TRUE;    // if the exclude pattern is empty, use the proxy
+      return true;    // if the exclude pattern is empty, use the proxy
 
-   return FALSE;
+   return false;
     
 }
 
