@@ -2,7 +2,7 @@
    //
    //    htcheck.h
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Some Portions Copyright (c) 1995-2000 The ht://Dig Group <www.htdig.org>
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
@@ -10,7 +10,7 @@
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: htcheck.h,v 1.4 2002/02/06 11:04:34 angusgb Exp $
+   //    $Id: htcheck.h,v 1.7 2003/12/30 09:38:47 angusgb Exp $
    //
    //
 ///////
@@ -18,6 +18,17 @@
 #ifndef _htcheck_h_
 #define _htcheck_h_
 
+#ifdef HAVE_STD
+#include <iostream>
+#include <fstream>
+#ifdef HAVE_NAMESPACES
+using namespace std;
+#endif
+#else
+#include <iostream.h>
+#include <fstream.h>
+#endif /* HAVE_STD */
+
 #include <Configuration.h>
 #include <List.h>
 #include <StringList.h>
@@ -25,8 +36,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <fstream.h>
-#include <iostream.h>
 #include <htconfig.h>
 #include <HtmysqlDB.h>
 #include "HtRegex.h"
