@@ -4,19 +4,23 @@
    //
    //    Class for Html statements
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
    //    For copyright details, see the file COPYING in your distribution
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: HtmlAttribute.cc,v 1.4 2002/02/18 10:44:59 angusgb Exp $
+   //    $Id: HtmlAttribute.cc,v 1.7 2003/12/30 09:38:47 angusgb Exp $
    //
    //    G.Bartolini
    //    started: 05.10.1999
 ///////
 
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "HtmlAttribute.h"
 
    // Static variables initialization
