@@ -4,19 +4,23 @@
    //
    //    Class for Html statements
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
    //    For copyright details, see the file COPYING in your distribution
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: HtmlStatement.cc,v 1.4 2002/02/18 10:44:59 angusgb Exp $
+   //    $Id: HtmlStatement.cc,v 1.9 2003/12/30 09:38:47 angusgb Exp $
    //
    //    G.Bartolini
    //    started: 05.10.1999
 ///////
 
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "HtmlStatement.h"
 
    // Static variables initialization
@@ -27,8 +31,13 @@
 ///////
 
 HtmlStatement::HtmlStatement()
+: IDUrl(0),
+   TagPosition(0),
+   Tag(0),
+   Statement(0),
+   Row(1),
+   LinkTagPosition(0)
 {
-   Reset();
 }
 
 ///////
@@ -49,6 +58,8 @@ void HtmlStatement::Reset()
    TagPosition = 0;
    Tag = 0;
    Statement = 0;
+   Row = 1;
+   LinkTagPosition = 0;
 }
 
 
@@ -65,7 +76,8 @@ ostream& operator<<(ostream& output, const HtmlStatement& s)
    
    // Only if debug level is greater than 2
    
-   output << " (Tag : " << s.Tag << " - Statement <"
+   output << " (Tag : " << s.Tag << " Row: " << s.Row
+      << " - Statement <"
       << s.Statement << ">)";
    
    return output;
