@@ -5,19 +5,23 @@
    //    Class to that contains all the general info about the run
    //    They will be stored into the 'htCheck' table of the database
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
    //    For copyright details, see the file COPYING in your distribution
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: RunInfo.cc,v 1.4 2002/02/06 11:04:34 angusgb Exp $
+   //    $Id: RunInfo.cc,v 1.8 2004/04/05 12:39:40 angusgb Exp $
    //
    //    G.Bartolini
    //    started: 18.08.2000
 ///////
 
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "RunInfo.h"
 
 ///////
@@ -25,15 +29,10 @@
 ///////
 
 RunInfo::RunInfo ()
+: RetrievedUrls(0), TotUrls(0), ScheduledUrls(0), HTTPSeconds(0),
+   HTTPRequests(0), HTTPBytes(0), TCPConnections(0), ServerChanges(0),
+   AccessibilityChecks(1)
 {
-   RetrievedUrls=0;
-   TotUrls=0;
-   ScheduledUrls=0;
-   HTTPSeconds=0;
-   HTTPRequests=0;
-   HTTPBytes=0;
-   TCPConnections=0;
-   ServerChanges=0;
 }
 
 ///////
