@@ -5,19 +5,19 @@
 //         This includes the ability to be put into a list
 //
 // Part of the ht://Dig package   <http://www.htdig.org/>
-// Copyright (c) 1999 The ht://Dig Group
+// Copyright (c) 1999, 2000 The ht://Dig Group
 // For copyright details, see the file COPYING in your distribution
-// or the GNU Public License version 2 or later 
+// or the GNU General Public License version 2 or later 
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: Object.h,v 1.1.1.1 2000/05/08 11:14:52 angusgb Exp $
+// $Id: Object.h,v 1.2 2002/11/14 17:09:01 angusgb Exp $
 //
 
 #ifndef	_Object_h_
 #define	_Object_h_
 
 #include "lib.h"
-#include <iostream.h>
+#include <stdio.h>
 
 class String;
 
@@ -34,13 +34,13 @@ public:
 	// To ensure a consistent comparison interface and to allow comparison
 	// of all kinds of different objects, we will define a comparison functions.
 	//
-	virtual int	compare(const Object &)	{ return 0;}
+	virtual int	compare(const Object &)	const { return 0;}
 
 	//
 	// To allow a deep copy of data structures we will define a standard interface...
 	// This member will return a copy of itself, freshly allocated and deep copied.
 	//
-	virtual Object	*Copy() const { cerr << "Object::Copy: derived class does not implement Copy\n"; return new Object(); }
+	virtual Object	*Copy() const { fprintf(stderr, "Object::Copy: derived class does not implement Copy\n"); return new Object(); }
 };
 
 
