@@ -2,22 +2,28 @@
    //    MySQL Database class for ht://Check
    //    File: HtmysqlDB.cc
    //
-   //    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+   //    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
    //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
    //
    //    For copyright details, see the file COPYING in your distribution
    //    or the GNU General Public License version 2 or later 
    //    <http://www.gnu.org/copyleft/gpl.html>
    //
-   //    $Id: HtmysqlDB.cc,v 1.46 2002/02/18 10:44:59 angusgb Exp $
+   //    $Id: HtmysqlDB.cc,v 1.76 2004/05/26 08:34:10 angusgb Exp $
    //
    //    Started: 28.06.1999
 ///////
 
-#include "HtmysqlDB.h"
 #include <stdlib.h>
 #include <ctype.h>
 
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
+#include "HtmysqlDB.h"
+#include "StringList.h"
+
 // Local function (declared as static)
 static bool getmysqlconfvalue(const char *source, const char *pattern,
    String &destination);
@@ -233,23 +239,41 @@ int HtmysqlDB::CreateDatabase()
    
    // Database already exists
    // Let's drop it before creating a new one with the same name
-   if (result > 0) DropDatabase();
+   if (result > 0)
+   {
+      if (drop_database)
+         DropDatabase(); // Drop the database
+      else
+      {
+         // Keep the existant database, but drop the tables
+         DropTables();
 
-
-///////
-   //    Database creation
-///////
+         if (debug >0)
+            cout << "Keep alive previous ht://Check database '" << MySQLDB << "'" << endl;
    
-   if (debug >0)
-      cout << "Creating ht://Check database '" << MySQLDB << "'" << endl;
-   
-   SQLStatement << "CREATE DATABASE " << MySQLDB;
-   
-   // Executing Database creation 
+      }
+   }
+   else
+      drop_database = true; // Database has been virtually 'dropped' (did not exist)
 
-   if (Query (SQLStatement) == -1)
-      return 0;   // An error occured
 
+   // If a database was not found or it has already dropped, we have to create it
+   if (drop_database)
+   {
+      ///////
+         //    Database creation
+      ///////
+   
+      if (debug >0)
+         cout << "Creating ht://Check database '" << MySQLDB << "'" << endl;
+  
+      SQLStatement << "CREATE DATABASE " << MySQLDB;
+   
+      // Executing Database creation 
+
+      if (Query (SQLStatement) == -1)
+         return 0;   // An error occured
+   }
    
    // Select the database
    SelectDB (MySQLDB);
@@ -279,6 +303,7 @@ int HtmysqlDB::CreateDatabase()
       << "  HTTPRequests MEDIUMINT UNSIGNED  DEFAULT '0' NOT NULL, " << "\n"
       << "  HTTPSeconds MEDIUMINT UNSIGNED  DEFAULT '0' NOT NULL, " << "\n"
       << "  HTTPBytes BIGINT UNSIGNED  DEFAULT '0' NOT NULL, " << "\n"
+      << "  AccessibilityChecks TINYINT UNSIGNED  DEFAULT '1' NOT NULL, " << "\n"
       << "  User VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
       << "  PRIMARY KEY (StartTime, EndTime)" << "\n"
       << ")" << "\n";
@@ -309,6 +334,7 @@ int HtmysqlDB::CreateDatabase()
       << " 'FileProtocol', 'EMail', 'Javascript', 'NotValidService',"
       << " 'Malformed') "
              << "DEFAULT 'ToBeRetrieved' NOT NULL, " << "\n"
+      << "  Domain ENUM('Internal', 'External') DEFAULT NULL," << "\n"
       << "  CreationTime DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL, " << "\n"
       << "  IDReferer MEDIUMINT UNSIGNED DEFAULT '0' NOT NULL, " << "\n"
       << "  HopCount TINYINT UNSIGNED DEFAULT '0' NOT NULL, " << "\n"
@@ -364,6 +390,13 @@ int HtmysqlDB::CreateDatabase()
 
 
    // Write the SQL statement
+
+   // First builds the Charset field as an enumeration of possible values
+   String tmpcharsets;
+   tmpcharsets.trunc();
+   AvailableCharsets.Start_Get();
+   while (char *cs = AvailableCharsets.Get_Next())
+      tmpcharsets << "'" << cs << "', ";
    
    SQLStatement.trunc(); // Initialize it again.
    SQLStatement << "CREATE TABLE " << MySQLDB << ".Url (" << "\n"
@@ -373,7 +406,7 @@ int HtmysqlDB::CreateDatabase()
       << "  ContentType VARCHAR(32) DEFAULT '' NOT NULL," << "\n"
       << "  ConnStatus ENUM('OK', 'NoHeader', 'NoHost', 'NoPort', "
          << "'NoConnection', 'ConnectionDown', 'ServiceNotValid', "
-         << "'OtherError') "
+         << "'OtherError', 'ServerError') "
          << "DEFAULT 'OK' NOT NULL, " << "\n"
       << "  ContentLanguage VARCHAR(16) DEFAULT '' NOT NULL," << "\n"
       << "  TransferEncoding VARCHAR(32) DEFAULT '' NOT NULL," << "\n"
@@ -384,12 +417,28 @@ int HtmysqlDB::CreateDatabase()
       << "  ReasonPhrase VARCHAR(32) DEFAULT '' NOT NULL," << "\n"
       << "  Location VARCHAR(255) BINARY DEFAULT '' NOT NULL," << "\n"
       << "  Title VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
+      << "  Contents MEDIUMTEXT DEFAULT NULL," << "\n"
+      << "  DocType ENUM('not-public', 'not-html', " << "\n"
+	  	<< " 'xhtml-11', 'xhtml-10', 'xhtml-10-transitional', 'xhtml-10-frameset'," << "\n"
+		<< " 'html-401', 'html-401-transitional', 'html-401-frameset'," << "\n"
+		<< " 'html-40', 'html-40-transitional', 'html-40-frameset'," << "\n"
+		<< " 'html-32', 'html-20', 'html-20-level2', 'html-20-level1'," << "\n"
+		<< " 'html-20-strict', 'html-20-strict-level1', 'html-iso-iec-15445-2000'," << "\n"
+		<< " 'unknown') DEFAULT NULL," << "\n"
+      << "  Charset ENUM(" << tmpcharsets << "'unknown') DEFAULT NULL," << "\n"
+      << "  Description VARCHAR(255) DEFAULT NULL," << "\n"
+      << "  Keywords VARCHAR(255) DEFAULT NULL," << "\n"
       << "  SizeAdd INT DEFAULT '0' NOT NULL," << "\n"
+      // << "  OutgoingLinks SMALLINT DEFAULT '0' NOT NULL," << "\n"
+      // << "  IncomingLinks SMALLINT DEFAULT '0' NOT NULL," << "\n"
+      // << "  IncomingUrls SMALLINT DEFAULT '0' NOT NULL," << "\n"
+      // << "  OutgoingDocuments SMALLINT DEFAULT '0' NOT NULL," << "\n"
       << "  PRIMARY KEY (IDUrl), " << "\n"
       << "  INDEX Idx_IDServer (IDServer), " << "\n"
       << "  " << SQL_Url_Index << ", " << "\n"
       << "  INDEX Idx_ContentType (ContentType(16)), " << "\n"
-      << "  INDEX Idx_StatusCode (StatusCode) " << "\n"
+      << "  INDEX Idx_StatusCode (StatusCode), " << "\n"
+      << "  INDEX Idx_Charset (Charset)" << "\n"
       << ")" ;
 
    // Executing table creation 
@@ -412,10 +461,14 @@ int HtmysqlDB::CreateDatabase()
    SQLStatement << "CREATE TABLE " << MySQLDB << ".HtmlStatement (" << "\n"
       << "  IDUrl MEDIUMINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
       << "  TagPosition SMALLINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
+      << "  Row MEDIUMINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
       << "  Tag VARCHAR(32) DEFAULT '' NOT NULL," << "\n"
       << "  Statement VARCHAR(255)," << "\n"
+      << "  LinkTagPosition SMALLINT UNSIGNED," << "\n"
+      << "  LinkDescription VARCHAR(255)," << "\n"
       << "  PRIMARY KEY (IDUrl, TagPosition)," << "\n"
-      << "  INDEX Idx_Tag (Tag(4))" << "\n"
+      << "  INDEX Idx_Tag (Tag(4))," << "\n"
+      << "  INDEX Idx_Statement (Tag(8))" << "\n"
       << ")" << "\n";
 
    // Executing table creation 
@@ -440,9 +493,10 @@ int HtmysqlDB::CreateDatabase()
       << "  TagPosition SMALLINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
       << "  AttrPosition TINYINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
       << "  Attribute VARCHAR(32) DEFAULT '' NOT NULL," << "\n"
-      << "  Content VARCHAR(255)," << "\n"
+      << "  Content VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
       << "  PRIMARY KEY (IDUrl, TagPosition, AttrPosition)," << "\n"
-      << "  INDEX Idx_Attribute (Attribute(8))" << "\n"
+      << "  INDEX Idx_Attribute (Attribute(8))," << "\n"
+      << "  INDEX Idx_Content (Content(8))" << "\n"
       << ")" << "\n";
 
    // Executing table creation 
@@ -473,8 +527,11 @@ int HtmysqlDB::CreateDatabase()
       << "  LinkResult "
          << "ENUM('NotChecked', 'NotRetrieved', 'OK', 'Broken', "
          << "'AnchorNotFound', 'Redirected', 'NotAuthorized', "
-	 << "'EMail', 'Javascript') "
+	 << "'EMail', 'Javascript', 'BadEncoded') "
          << "DEFAULT 'NotChecked' NOT NULL, " << "\n"
+      << "  LinkDomain "
+         << "ENUM('SameServer', 'Internal', 'External') "
+         << "DEFAULT NULL, " << "\n"
       << "  PRIMARY KEY (IDUrlSrc, IDUrlDest, TagPosition, AttrPosition)" << "\n"
       << ")" << "\n";
 
@@ -501,6 +558,8 @@ int HtmysqlDB::CreateDatabase()
       << "  Value TEXT DEFAULT '' NOT NULL," << "\n"
       << "  Path VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
       << "  Domain VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
+      << "  MaxAge MEDIUMINT DEFAULT '-1' NOT NULL," << "\n"
+      << "  Version TINYINT DEFAULT '0' NOT NULL," << "\n"
       << "  SrcUrl VARCHAR(255) DEFAULT '' NOT NULL," << "\n"
       << "  Expires DATETIME DEFAULT '0000-00-00 00:00:00' NOT NULL, " << "\n"
       << "  Secure TINYINT DEFAULT '0' NOT NULL, " << "\n"
@@ -514,6 +573,34 @@ int HtmysqlDB::CreateDatabase()
    if (Query (SQLStatement) == -1)
       return 0;   // An error occured
 
+///////
+   //    Creation of the 'Accessibility' table
+///////
+
+   if (debug >1)
+      cout << " |- Creating 'Accessibility' table" << endl;
+
+
+   // Write the SQL statement
+   
+   SQLStatement.trunc(); // Initialize it again.
+   SQLStatement << "CREATE TABLE " << MySQLDB << ".Accessibility (" << "\n"
+      << "  IDCheck MEDIUMINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
+      << "  IDUrl MEDIUMINT UNSIGNED DEFAULT '0' NOT NULL," << "\n"
+      << "  TagPosition SMALLINT UNSIGNED DEFAULT '0' NULL," << "\n"
+      << "  AttrPosition TINYINT UNSIGNED DEFAULT '0' NULL," << "\n"
+      << "  Code TINYINT UNSIGNED DEFAULT '0' NULL," << "\n"
+      << "  PRIMARY KEY (IDCheck)," << "\n"
+      << "  INDEX (IDUrl, TagPosition, AttrPosition)," << "\n"
+      << "  INDEX (Code, IDUrl, TagPosition)" << "\n"
+      << ")" << "\n";
+
+
+   // Executing table creation 
+
+   if (Query (SQLStatement) == -1)
+      return 0;   // An error occured
+
 ///////
    //    Creation of the 'TmpAnchors' temporary table
 ///////
@@ -569,6 +656,42 @@ int HtmysqlDB::DropDatabase()
 }
 
 
+///////
+   //    Keep the database for ht://Check, but recreate the tables
+   //    Return 0 if an error occured, 1 OK
+///////
+
+int HtmysqlDB::DropTables()
+{
+
+   String SQLStatement="";
+   
+   if (debug >0)
+      cout << "Dropping the tables of a previous ht://Check database called '"
+      << MySQLDB << "'" << endl;
+
+   SQLStatement << "DROP TABLE IF EXISTS " 
+      << MySQLDB << ".HtmlAttribute, " 
+      << MySQLDB << ".HtmlStatement, " 
+      << MySQLDB << ".Link, " 
+      << MySQLDB << ".Schedule, " 
+      << MySQLDB << ".Server, " 
+      << MySQLDB << ".Url, " 
+      << MySQLDB << ".htCheck, " 
+      << MySQLDB << ".TmpAnchors, " 
+      << MySQLDB << ".Cookies, "
+      << MySQLDB << ".Accessibility ";
+
+
+   // Executing Database dropping
+
+   if (Query (SQLStatement) == -1)
+      return 0;   // An error occured
+
+   return 1;
+}
+
+
 ///////
    //    Set the SQL BIG TABLES option (for huge queries)
 ///////
@@ -767,9 +890,13 @@ int HtmysqlDB::Insert(const _Url &url)
 
    String SQLStatement="";
    String Status="";
+   String DocType="";
    
    // Retrieve the String value of the Status
    url.RetrieveConnStatus(Status);   
+
+   // Retrieve the string value for the doctype
+   url.RetrieveDocType(DocType);   
    
    if (debug >4)
       cout << "Inserting a new url into '"
@@ -778,7 +905,9 @@ int HtmysqlDB::Insert(const _Url &url)
    SQLStatement << "Insert into " << MySQLDB << ".Url ( "
       << "IDUrl, IDServer, Url, ContentType, ConnStatus, TransferEncoding, "
       << "LastModified, LastAccess, Size, StatusCode, "
-      << "ReasonPhrase, Location, Title, ContentLanguage"
+      << "ReasonPhrase, Location, Title, ContentLanguage, "
+	  << "Contents, Charset, DocType, "
+	  << "Description, Keywords"
       << " ) values ( "
       << url.GetID() << ", "
       << url.GetIDServer() << ", ";
@@ -821,6 +950,53 @@ int HtmysqlDB::Insert(const _Url &url)
    // ContentLanguage
    AppendSQLTextField(SQLStatement, url.GetContentLanguage());
 
+   SQLStatement << ", ";
+
+   // Contents
+   if (url.GetContents())
+      AppendSQLTextField(SQLStatement, *(url.GetContents()));
+   else
+      SQLStatement << "NULL";
+
+   SQLStatement << ", ";
+
+   // Charset
+   if (url.GetCharset().length())
+   {
+      String cs = url.GetCharset();
+      cs.lowercase();
+      if (! AvailableCharsets.Exists(cs))
+         cs = "unknown";
+
+      AppendSQLTextField(SQLStatement, cs);
+   }
+   else
+      SQLStatement << "NULL";
+
+   SQLStatement << ", ";
+
+   // DocType
+   if (DocType.length())
+      AppendSQLTextField(SQLStatement, DocType);
+   else
+      SQLStatement << "NULL";
+
+   SQLStatement << ", ";
+
+   // Description
+   if (url.GetDescription().length())
+      AppendSQLTextField(SQLStatement, url.GetDescription());
+   else
+      SQLStatement << "NULL";
+
+   SQLStatement << ", ";
+
+   // Keywords
+   if (url.GetKeywords().length())
+      AppendSQLTextField(SQLStatement, url.GetKeywords());
+   else
+      SQLStatement << "NULL";
+
    SQLStatement << ")";
 
    // Executing Insert query
@@ -842,16 +1018,18 @@ int HtmysqlDB::Insert(SchedulerEntry &s)
 
    String SQLStatement="";
    String Status="";
+   String Domain="";
 
-   // Retrieve the String value of the Status
+   // Retrieve the String value of the Status and the domain
    s.RetrieveStatus(Status);   
+   s.RetrieveDomain(Domain);   
    
    if (debug >4)
       cout << "Inserting a new scheduler entry into '"
       << MySQLDB << "': " << s << endl;
 
    SQLStatement << "Insert into " << MySQLDB << ".Schedule ( "
-      << "IDUrl, IDServer, Url, CreationTime, Status, IDReferer, HopCount"
+      << "IDUrl, IDServer, Url, CreationTime, Status, IDReferer, HopCount, Domain"
       << " ) values ( "
       << s.GetIDSchedule() << ", "
       << s.GetIDServer() << ", ";
@@ -862,17 +1040,18 @@ int HtmysqlDB::Insert(SchedulerEntry &s)
       << "NOW(), "
       << "'" << Status << "', "
       << s.GetIDReferer() << ", "
-      << s.GetHopCount() << " "
-      << ")";
+      << s.GetHopCount() << ", ";
+
+   if (Domain.length())
+      SQLStatement << "'" << Domain << "')";
+   else
+      SQLStatement << "NULL" << ")";
 
    // Executing Insert query
 
    if (Query (SQLStatement) == -1)
       return 0;   // An error occured
 
-   // Retrieve the ID (cos it's auto incremental)
-   s.SetIDSchedule(GetLastID());
-   
    return 1;
 }
 
@@ -893,7 +1072,7 @@ int HtmysqlDB::Insert(const RunInfo &runinfo)
    SQLStatement << "Insert into " << MySQLDB << ".htCheck ( "
       << "StartTime, EndTime, ScheduledUrls, TotUrls, "
       << "RetrievedUrls, TCPConnections, ServerChanges, "
-      << "HTTPRequests, HTTPSeconds, HTTPBytes, User"
+      << "HTTPRequests, HTTPSeconds, HTTPBytes, AccessibilityChecks, User"
       << " ) values ( "
       << "'" << runinfo.StartTime.GetTimeStamp() << "', "
       << "'" << runinfo.FinishTime.GetTimeStamp() << "', "
@@ -905,6 +1084,7 @@ int HtmysqlDB::Insert(const RunInfo &runinfo)
       << runinfo.HTTPRequests << ", "
       << runinfo.HTTPSeconds << ", "
       << runinfo.HTTPBytes << ", "
+      << runinfo.AccessibilityChecks << ", "
       << "USER()"
       << ")";
 
@@ -935,7 +1115,7 @@ int HtmysqlDB::Insert(const HtCookie &cookie)
       << MySQLDB << "': " << cookie.GetName() << endl;
 
    SQLStatement << "Insert into " << MySQLDB << ".Cookies ( "
-      << "IDCookie, Name, Value, Path, Domain, SrcUrl, Secure, DomainValid";
+      << "IDCookie, Name, Value, Path, Domain, MaxAge, Version, SrcUrl, Secure, DomainValid";
    
    if (cookie.GetExpires())
       SQLStatement << ", Expires";
@@ -958,9 +1138,18 @@ int HtmysqlDB::Insert(const HtCookie &cookie)
 
    SQLStatement << ", ";
 
-   // Escape safe cookie path
+   // Escape safe cookie domain
    AppendSQLTextField(SQLStatement, cookie.GetDomain());
 
+
+   // Insert the max-age attribute
+   SQLStatement << ", "
+      << cookie.GetMaxAge();
+
+   // Insert the max-age attribute
+   SQLStatement << ", "
+      << cookie.GetVersion();
+
    SQLStatement << ", ";
 
    // Escape safe cookie source URL
@@ -1221,9 +1410,10 @@ int HtmysqlDB::Insert(const HtmlStatement& htmlstatement)
       << MySQLDB << "': " << htmlstatement << endl;
 
    SQLStatement << "Insert into " << MySQLDB << ".HtmlStatement ( "
-      << "IDUrl, TagPosition, Tag, Statement"
+      << "IDUrl, Row, TagPosition, Tag, Statement, LinkTagPosition"
       << " ) values ( "
       << htmlstatement.GetIDUrl() << ", "
+      << htmlstatement.GetRow() << ", "
       << htmlstatement.GetTagPosition() << ", ";
 
    AppendSQLTextField(SQLStatement, htmlstatement.GetTag());
@@ -1232,6 +1422,14 @@ int HtmysqlDB::Insert(const HtmlStatement& htmlstatement)
 
    AppendSQLTextField(SQLStatement, htmlstatement.GetStatement());
 
+   SQLStatement << ", ";
+
+   if (htmlstatement.GetTagPosition() != htmlstatement.GetLinkTagPosition()
+      && htmlstatement.GetLinkTagPosition() > 0)
+         SQLStatement << htmlstatement.GetLinkTagPosition();
+   else
+	  SQLStatement << "NULL";
+
    SQLStatement << ")";
 
    // Executing Insert query
@@ -1243,6 +1441,37 @@ int HtmysqlDB::Insert(const HtmlStatement& htmlstatement)
 }
 
 
+///////
+   //    Insert a Link description into the HtmlStatement table
+   //    Returns 0 if an error occured
+///////
+
+int HtmysqlDB::InsertHtmlStatementLinkDescription(const unsigned int IDUrl,
+    const unsigned int TagPosition, const String& LinkDescription)
+{
+
+   String SQLStatement="";
+   
+   if (debug >4)
+      cout << "Inserting a new Link description into '"
+      << MySQLDB << "': tag " << TagPosition << endl;
+
+   SQLStatement << "Update " << MySQLDB << ".HtmlStatement "
+      << " SET LinkDescription=";
+
+   AppendSQLTextField(SQLStatement, LinkDescription);
+   
+   SQLStatement << " WHERE IDUrl=" << IDUrl << " AND TagPosition=" << TagPosition;
+
+   // Executing Insert query
+
+   if (Query (SQLStatement) == -1)
+      return 0;   // An error occured
+
+   return 1;
+}
+
+
 ///////
    //    Insert a HtmlAttribute into the relative table
    //    Returns 0 if an error occured
@@ -1281,6 +1510,49 @@ int HtmysqlDB::Insert(const HtmlAttribute& htmlattribute)
 }
 
 
+///////
+   //    Insert a AccessibilityCheck into the relative table
+   //    Returns 0 if an error occured
+///////
+
+int HtmysqlDB::Insert(const AccessibilityCheck& accessibilitycheck)
+{
+
+   String SQLStatement="";
+   
+   if (debug >4)
+      cout << "Inserting a new Accessibility into '"
+      << MySQLDB << "': " << accessibilitycheck << endl;
+
+   SQLStatement << "Insert into " << MySQLDB << ".Accessibility ( "
+      << "IDCheck, IDUrl, TagPosition, AttrPosition, Code"
+      << " ) values ( "
+      << accessibilitycheck.GetIDCheck() << ", "
+      << accessibilitycheck.GetIDUrl() << ", ";
+
+   if (accessibilitycheck.GetTagPosition() == 0)
+      SQLStatement << "NULL";
+   else
+      SQLStatement << accessibilitycheck.GetTagPosition();
+
+   SQLStatement << ", ";
+
+   if (accessibilitycheck.GetAttrPosition() == 0)
+      SQLStatement << "NULL";
+   else
+      SQLStatement << accessibilitycheck.GetAttrPosition();
+
+   SQLStatement << ", " << accessibilitycheck.GetCode() << ")";
+      
+   // Executing Insert query
+
+   if (Query (SQLStatement) == -1)
+      return 0;   // An error occured
+
+   return 1;
+}
+
+
 ///////
    //    Insert a Link into the relative table
    //    Returns 0 if an error occured
@@ -1292,6 +1564,7 @@ int HtmysqlDB::Insert(const Link& link)
    String SQLStatement="";
    String LinkType="";
    String LinkResult="";
+   String LinkDomain="";
    
    // Retrieve the String value of the type
    link.RetrieveLinkType(LinkType);   
@@ -1299,6 +1572,9 @@ int HtmysqlDB::Insert(const Link& link)
    // Retrieve the String value of the result
    link.RetrieveLinkResult(LinkResult);   
    
+   // Retrieve the String value of the domain
+   link.RetrieveLinkDomain(LinkDomain);   
+   
    
    if (debug >4)
       cout << "Inserting a new Link into '"
@@ -1306,7 +1582,7 @@ int HtmysqlDB::Insert(const Link& link)
 
    SQLStatement << "Insert into " << MySQLDB << ".Link ( "
       << "IDUrlSrc, IDUrlDest, TagPosition, AttrPosition, "
-      << "Anchor, LinkType, LinkResult"
+      << "Anchor, LinkType, LinkResult, LinkDomain"
       << " ) values ( "
       << link.GetIDUrlSrc() << ", "
       << link.GetIDUrlDest() << ", "
@@ -1317,8 +1593,12 @@ int HtmysqlDB::Insert(const Link& link)
 
    SQLStatement << ", "
       << "'" << LinkType << "', "
-      << "'" << LinkResult << "'"
-      << ")";
+      << "'" << LinkResult << "', ";
+      
+   if (LinkDomain.length())
+      SQLStatement << "'" << LinkDomain << "')";
+   else
+      SQLStatement << "NULL" << ")";
 
    // Executing Insert query
 
@@ -1434,7 +1714,8 @@ int HtmysqlDB::CreateAnchorsTable()
       << "HtmlAttribute.AttrPosition, HtmlAttribute.Content FROM HtmlAttribute, "
       << "HtmlStatement WHERE HtmlStatement.IDUrl=HtmlAttribute.IDUrl AND "
       << "HtmlAttribute.TagPosition=HtmlStatement.TagPosition AND "
-      << "HtmlAttribute.Attribute='name' AND HtmlStatement.Tag='a'";
+      << "((HtmlAttribute.Attribute='name' AND HtmlStatement.Tag='a') "
+      << "OR HtmlAttribute.Attribute='id')";
 
    if (debug>2)
       cout << "Filling temporary table TmpAnchors" << endl;
@@ -1446,6 +1727,91 @@ int HtmysqlDB::CreateAnchorsTable()
 }
 
 
+// Create and fill the temporary table for managing anchors
+
+int HtmysqlDB::AnchorsTable(ostream& output)
+{
+   String SQLStatement=0;
+   Link LinkTmp;  // Temporary link object
+
+   // Create the temporary table
+   CreateAnchorsTable();
+
+   // Now the temporary table is filled with all the anchors found
+
+   SQLStatement << "SELECT DISTINCT Link.IDUrlSrc, Link.IDUrlDest, "
+      << "Link.TagPosition, Link.AttrPosition, Link.Anchor, TmpAnchors.Anchor "
+      << "FROM Link, Schedule LEFT JOIN TmpAnchors ON "
+      << "TmpAnchors.IDUrl=Link.IDUrlDest "
+      << "AND TmpAnchors.Anchor=Link.Anchor "
+      << "WHERE Link.Anchor != '' AND LCASE(Link.Anchor) != 'top' "
+      << "AND Link.LinkResult='OK' "
+      << "AND Link.IDUrlDest=Schedule.IDUrl AND Schedule.Status='Retrieved'";
+  
+   HtmysqlQueryResult ResultTmp;
+   
+   int NumAnchors = 0;
+   int NumAnchorsNotFound = 0;
+
+   if (Query (SQLStatement, &ResultTmp) == -1)
+      return -1;
+
+   MYSQL_ROW LinkRow;
+   
+   while ((LinkRow = (ResultTmp.GetNextRecord())))
+   {
+      ++NumAnchors;
+      
+      if (! LinkRow[5])
+      {
+         // Anchor Not found
+         
+         ++NumAnchorsNotFound;
+         LinkTmp.Reset();  
+         LinkTmp.SetIDUrlSrc((unsigned int) atoi(LinkRow[0]));
+         LinkTmp.SetIDUrlDest((unsigned int) atoi(LinkRow[1]));
+         LinkTmp.SetTagPosition((unsigned int) atoi(LinkRow[2]));
+         LinkTmp.SetAttrPosition((unsigned int) atoi(LinkRow[3]));
+         LinkTmp.SetAnchor(LinkRow[4]);
+
+         LinkTmp.SetLinkResult(Link::Link_AnchorNotFound);
+         
+         if (debug>2)
+               output << "Anchor not found: " << LinkTmp
+                  << "#" << LinkTmp.GetAnchor() << endl;
+
+         // Update the link table - setting the results
+
+         if(SetAnchorsResults(LinkTmp) == -1) // A database error occured
+            return -1;
+      }
+
+   }
+
+   ResultTmp.Free();      
+
+   if (debug>0)
+      output << "Anchors checked: " << NumAnchors
+         << " - Not found: " << NumAnchorsNotFound << endl;
+
+
+///////
+   //    Drop the 'TmpAnchors' temporary table
+///////
+
+   if (debug >1)
+      output << " |- Dropping 'TmpAnchors' temporary table" << endl;
+
+   // Write the SQL statement
+   SQLStatement.trunc();
+   SQLStatement << "DROP TABLE " << MySQLDB << ".TmpAnchors";
+
+   // Executing table dropping
+   return Query (SQLStatement);
+
+}
+
+
 // Set the result type for anchors
 
 int HtmysqlDB::SetAnchorsResults(Link &LinkTmp)
@@ -1468,6 +1834,144 @@ int HtmysqlDB::SetAnchorsResults(Link &LinkTmp)
 }
 
 
+///////
+   //    Set the results for the links. It is called by the Scheduler at the end
+   //    of the crawl.
+   //    Look for all the links that have 'NotChecked' flag in the LinkResult
+   //    field. For each of them, it checks all of the links, and see if
+   //    they are eithrt found or not retrieved or broken or redirected.
+   //    Then it updates the Link table with the right flag.
+   //    Now the BadEncoded case is treated as well.
+///////
+
+
+int HtmysqlDB::SetLinkResults(ostream& output)
+{
+
+   // Set the link results
+
+   String SQLStatement=0;
+   Link LinkTmp;  // Temporary link object
+   
+   SQLStatement << "SELECT distinct IDUrlSrc, IDUrlDest, "
+      << "Url.StatusCode, Link.LinkResult "
+      << "FROM Link LEFT JOIN Url on Link.IDUrlDest=Url.IDUrl "
+      << "WHERE Link.LinkResult='NotChecked' or Link.LinkResult='BadEncoded'";
+
+   HtmysqlQueryResult ResultTmp;
+   
+   int NumLinks = 0;
+
+   if (debug>0)
+      output << endl << "Setting link results" << endl;
+
+   if (Query (SQLStatement, &ResultTmp) == -1)
+      return -1;
+
+   String strResult;
+   MYSQL_ROW LinkRow;
+   
+   while ((LinkRow = (ResultTmp.GetNextRecord())))
+   {
+      unsigned int StatusCode;
+      
+      ++NumLinks;
+      
+      LinkTmp.Reset();  
+      LinkTmp.SetIDUrlSrc((unsigned int) atoi(LinkRow[0]));
+      LinkTmp.SetIDUrlDest((unsigned int) atoi(LinkRow[1]));
+      LinkTmp.SetLinkResult(LinkRow[3]);
+
+      if (LinkRow[2])
+      {
+         StatusCode=(unsigned int) atoi(LinkRow[2]);
+         if (StatusCode == 200)
+         {
+            // Found - Let's check whether we had yet labeled it as 'BadEncoded' or not
+            if (LinkTmp.GetLinkResult() != Link::Link_BadEncoded)
+               LinkTmp.SetLinkResult(Link::Link_OK);  // No
+         
+         if (debug>2)
+               output << "Link found: " << LinkTmp << endl;
+         }
+         else if (StatusCode >= 300 && StatusCode < 400)
+         {
+            // Redirected
+            if (LinkTmp.GetLinkResult() != Link::Link_BadEncoded)
+               LinkTmp.SetLinkResult(Link::Link_Redirected);
+         
+         if (debug>2)
+               output << "Link redirected: " << LinkTmp << endl;
+         }
+         else if (StatusCode == 401)
+         {
+            // Not autorized
+            if (LinkTmp.GetLinkResult() != Link::Link_BadEncoded)
+               LinkTmp.SetLinkResult(Link::Link_NotAuthorized);
+         
+         if (debug>2)
+               output << "Link not authorized: " << LinkTmp << endl;
+         }
+         else
+         {
+            // Broken
+            LinkTmp.SetLinkResult(Link::Link_Broken);
+         
+         if (debug>2)
+               output << "Link broken: " << LinkTmp << endl;
+         }
+      }
+      else
+      {
+         if (LinkTmp.GetLinkResult() != Link::Link_BadEncoded)
+            LinkTmp.SetLinkResult(Link::Link_NotRetrieved); // Not Retrieved
+      }
+      
+      LinkTmp.RetrieveLinkResult(strResult);
+
+      String SQLUpdateStatement=0;
+      SQLUpdateStatement << "Update Link SET LinkResult = '" << strResult << "'"
+         << " WHERE IDUrlSrc = " << LinkTmp.GetIDUrlSrc()
+         << " AND IDUrlDest = " << LinkTmp.GetIDUrlDest()
+         << " AND LinkResult = '" << LinkRow[3] << "'";
+
+      if(Query(SQLUpdateStatement) == -1) // A database error occured
+         return -1;
+
+   }
+
+   ResultTmp.Free();      
+   if (debug>0)
+      output << "Links checked: " << NumLinks << endl;
+
+   return 0;
+   
+}
+
+
+///////
+   //    Set the list of available charsets
+///////
+
+void HtmysqlDB::LoadAvailableCharsets(const String& c)
+{
+    if (debug>0)
+        cout << "Loading available charsets: ";
+    StringList charsets(c, " \t");
+
+    for (int i = 0; i < charsets.Count(); i++)
+    {
+        if (debug>0)
+            cout << charsets[i] << " ";
+
+	AvailableCharsets.Add(charsets[i], (Object*)0);
+    }
+
+    if (debug>0)
+        cout << endl;
+}
+
+
 #if 0
 
 ///////
