@@ -7,12 +7,12 @@
 //             mixed in...
 //
 // Part of the ht://Dig package   <http://www.htdig.org/>
-// Copyright (c) 1999-2001 The ht://Dig Group
+// Copyright (c) 1999-2003 The ht://Dig Group
 // For copyright details, see the file COPYING in your distribution
 // or the GNU Public License version 2 or later 
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: Connection.cc,v 1.7 2002/02/07 07:57:39 angusgb Exp $
+// $Id: Connection.cc,v 1.11 2003/06/20 16:47:30 mnencia Exp $
 //
 #ifdef HAVE_CONFIG_H
 #include "htconfig.h"
@@ -26,23 +26,54 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>	// For inet_ntoa
+#endif /* HAVE_ARPA_INET_H */
+
 #include <netinet/in.h>
+
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
+#endif /* HAVE_SYS_IOCTL_H */
+
 #include <sys/uio.h>
+
+#ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
+#endif /* HAVE_SYS_FILE_H */
+
 #include <signal.h>
 #include <unistd.h>
+
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif /* HAVE_ARPA_INET_H */
+
 #include <netdb.h>
 #include <stdlib.h>
-#include <sys/time.h>
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <unistd.h>
+
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
-#endif
+#endif /* HAVE_STRINGS_H */
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif
+#endif /* HAVE_SYS_SELECT_H */
+
 
 typedef void (*SIGNAL_HANDLER) (...);
 
@@ -68,7 +99,7 @@ List	all_connections;
 //*************************************************************************
 Connection::Connection(int socket)
 : pos(0), pos_max(0),
-   sock(socket), connected(0), peer(""), server_name(""),
+   sock(socket), connected(0), peer(""), server_name(""), server_ip_address(""),
    need_io_stop(0), timeout_value(0), retry_value(1),
    wait_time(5) // wait 5 seconds after a failed connection attempt
 {
@@ -86,7 +117,7 @@ Connection::Connection(int socket)
 Connection::Connection(const Connection& rhs)
 : pos(rhs.pos), pos_max(rhs.pos_max),
    sock(rhs.sock), connected(rhs.connected),
-   peer(rhs.peer), server_name(rhs.server_name),
+   peer(rhs.peer), server_name(rhs.server_name), server_ip_address(rhs.server_ip_address),
    need_io_stop(rhs.need_io_stop), timeout_value(rhs.timeout_value),
    retry_value(rhs.retry_value),
    wait_time(rhs.wait_time) // wait 5 seconds after a failed connection attempt
@@ -218,13 +249,13 @@ int Connection::Assign_Server(unsigned int addr)
     return OK;
 }
 
-extern "C" unsigned int   inet_addr(char *);
 
 //*****************************************************************************
 //
 int Connection::Assign_Server(const String& name)
 {
     struct hostent *hp;
+    char **alias_list;
     unsigned int addr;
 
     //
@@ -233,19 +264,22 @@ int Connection::Assign_Server(const String& name)
     addr = inet_addr((char*)name.get());
     if (addr == (unsigned int)~0)
     {
-	hp = gethostbyname(name);
-	if (hp == 0)
-	{
-	    return NOTOK;
-	}
-	memcpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
+        // Gets the host given a string
+        hp = gethostbyname(name);
+
+        if (hp == 0)
+           return NOTOK;
+
+        alias_list = hp->h_aliases;
+        memcpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
     }
     else
     {
-	memcpy((char *)&server.sin_addr, (char *)&addr, sizeof(addr));
+        memcpy((char *)&server.sin_addr, (char *)&addr, sizeof(addr));
     }
 
     server_name = name.get();
+    server_ip_address = inet_ntoa(server.sin_addr);
 
     return OK;
 }
