@@ -8,12 +8,12 @@
 // 	 -  Response message class
 //
 // Part of the ht://Dig package   <http://www.htdig.org/>
-// Copyright (c) 1995-2001 The ht://Dig Group
+// Copyright (c) 1995-2003 The ht://Dig Group
 // For copyright details, see the file COPYING in your distribution
 // or the GNU Public License version 2 or later
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: HtHTTP.cc,v 1.21 2002/02/07 07:57:39 angusgb Exp $ 
+// $Id: HtHTTP.cc,v 1.30 2003/12/21 08:32:54 angusgb Exp $ 
 //
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,16 @@
 #include <sys/types.h>
 #include <ctype.h>
 #include <stdio.h>      // for sscanf
-#include <iomanip.h>    // for setw()
+
+// for setw()
+#ifdef HAVE_STD
+#include <iomanip>
+#ifdef HAVE_NAMESPACES
+using namespace std;
+#endif
+#else
+#include <iomanip.h>
+#endif /* HAVE_STD */
 
 #if 1
 typedef void (*SIGNAL_HANDLER) (...);
@@ -113,7 +122,6 @@ HtHTTP::HtHTTP(Connection& connection)
 : Transport(&connection),
    _Method(Method_GET), // Default Method Request
    _bytes_read(0),
-   _useproxy(0),
    _accept_language(0),
    _persistent_connection_allowed(true),
    _persistent_connection_possible(false),
@@ -176,12 +184,27 @@ Transport::DocStatus HtHTTP::Request()
       CloseConnection();      // Close a previous connection
  
       if (debug>0)
-         cout << "! Impossible to get the HTTP header line." << endl
-              << "  Connection closed. Try to get it again." << endl;
+      {
+          cout << "  # -> connection closed (try again)" << endl;
+          if (debug>1)
+            cout << "! Impossible to get the HTTP header line." << endl;
+      }
  
       result = HTTPRequest(); // Get the document again
  
    }
+   else if(result == Document_server_error && _Method == Method_HEAD)
+   {
+       // In some cases, when performing a HEAD request some server
+       // may respond with an internal server error; the library will
+       // automatically recover, issuing a GET request
+
+      if (debug>0)
+          cout << "  # -> HEAD request failed (try with GET)" << endl;
+ 
+      _Method = Method_GET;
+      result = HTTPRequest();
+   }
 
    return result;
 }
@@ -590,6 +613,16 @@ void HtHTTP::SetRequestCommand(String &cmd)
    if (_credentials.length())
      cmd << "Authorization: Basic " << _credentials << "\r\n";
 
+   // Proxy Authentication
+   if (_useproxy && _proxy_credentials.length())
+     cmd << "Proxy-Authorization: Basic " << _proxy_credentials << "\r\n";
+
+   // Accept-Encoding: waiting to handle the gzip and compress formats, we
+   // just send an empty header which, according to the HTTP 1/1 standard,
+   // should let the server know that we only accept the 'identity' case
+   // (no encoding of the document)
+   cmd << "Accept-Encoding: \r\n";
+
    // A date has been passed to check if the server one is newer than
    // the one we already own.
 
@@ -654,11 +687,11 @@ int HtHTTP::ParseHeader()
          // Status - Line check
          char	*token = line.get();
 
-         while (*token && !isspace(*token))
-            token++;
+         while (*token && !isspace(*token) && *token != ':')
+            ++token;
 
-         while (*token && isspace(*token))
-            token++;
+         while (*token && (isspace(*token) || *token == ':'))
+            ++token;
 	
          if(!strncmp((char*)line, "HTTP/", 5))
          {
@@ -772,7 +805,7 @@ int HtHTTP::ParseHeader()
  	    // Found a cookie
 		
             // Are cookies enabled?
-            if (_send_cookies)
+			if (_send_cookies && _cookie_jar)
             {
                token = strtok(token, "\n\t");
 
@@ -820,7 +853,7 @@ bool HtHTTP::isParsable(const char *content_type)
    // depending on the value of Transport:_default_parser_content_type
    // and the rest are determined by the external_parser settings
    
-   if( ! mystrncasecmp (_default_parser_content_type, content_type,
+   if( ! mystrncasecmp (_default_parser_content_type.get(), content_type,
       _default_parser_content_type.length()) )
        return true;
        
@@ -909,67 +942,25 @@ HtHTTP::DocStatus HtHTTP::GetDocumentStatus(HtHTTP_Response &r)
 	    returnStatus = Document_redirect;      	 // Redirection 3xx
    else if(statuscode==401)
 	    returnStatus = Document_not_authorized;   // Unauthorized
+   else if(statuscode >= 500 && statuscode < 600)
+	    returnStatus = Document_server_error;   // Internal server error 5xx
 
    // Exit the function
    return returnStatus;
 	    
 }
 
-
 void HtHTTP::SetCredentials (const String& s)
 {
-  // Overload default Transport method to take care of Basic HTTP auth.
-    static char	tbl[64] =
-    {
-	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
-	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
-	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
-	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
-	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
-	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
-	'w', 'x', 'y', 'z', '0', '1', '2', '3',
-	'4', '5', '6', '7', '8', '9', '+', '/'
-    };
-    _credentials = 0;
-    const char	*p;
-    int		n = s.length();
-    int		ch;
-
-    for (p = s.get(); n > 2; n -= 3, p += 3)
-    {
-	ch = *p >> 2;
-	_credentials << tbl[ch & 077];
-	ch = ((*p << 4) & 060) | ((p[1] >> 4) & 017);
-	_credentials << tbl[ch & 077];
-	ch = ((p[1] << 2) & 074) | ((p[2] >> 6) & 03);
-	_credentials << tbl[ch & 077];
-	ch = p[2] & 077;
-	_credentials << tbl[ch & 077];
-    }
-
-    if (n != 0)
-    {
-	char c1 = *p;
-	char c2 = n == 1 ? 0 : p[1];
-
-	ch = c1 >> 2;
-	_credentials << tbl[ch & 077];
-
-	ch = ((c1 << 4) & 060) | ((c2 >> 4) & 017);
-	_credentials << tbl[ch & 077];
-
-	if (n == 1)
-	    _credentials << '=';
-	else
-        {
-	    ch = (c2 << 2) & 074;
-	    _credentials << tbl[ch & 077];
-        }
-	_credentials << '=';
-    }
+   Transport::SetHTTPBasicAccessAuthorizationString(_credentials, s);
 }
 
 
+void HtHTTP::SetProxyCredentials (const String& s)
+{
+   Transport::SetHTTPBasicAccessAuthorizationString(_proxy_credentials, s);
+}
+
 int HtHTTP::ReadBody()
 {
 
