@@ -12,9 +12,14 @@
 // or the GNU Public License version 2 or later 
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: HtCodec.cc,v 1.1 2001/03/22 12:42:35 angusgb Exp $
+// $Id: HtCodec.cc,v 1.2 2002/11/14 16:59:05 angusgb Exp $
 //
 
+
+#ifdef HAVE_CONFIG_H
+#include "htconfig.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "HtCodec.h"
 
 HtCodec::HtCodec()
