@@ -5,37 +5,36 @@
 //
 //    Part of the ht://Check package
 //
-//    Copyright (c) 1999-2002 Comune di Prato - Prato - Italy
+//    Copyright (c) 1999-2004 Comune di Prato - Prato - Italy
 //    Author: Gabriele Bartolini - Prato - Italy <angusgb@users.sourceforge.net>
 //
 //    For copyright details, see the file COPYING in your distribution
 //    or the GNU General Public License version 2 or later 
 //    <http://www.gnu.org/copyleft/gpl.html>
 //
-//    $Id: footer.inc.php,v 1.1 2002/02/11 07:38:15 angusgb Exp $
+//    $Id: footer.inc.php,v 1.5 2003/12/30 09:39:23 angusgb Exp $
 
 if ( defined( '__FOOTER_INC' ) ) return;
 define( '__FOOTER_INC', 1 );
 
 ?>
 
-<br>
+<br style="clear: both">
+</div>
 
-<table border="0" width="100%" cellpadding="4" cellspacing="0" bgcolor="#ffffff">
-<tr>
+<div id="footer">
+<div class="toolbar">
 <?php
    if (!empty($linkbar))
-   { ?>
-<td align="leftt" valign="middle">
-<?php echo $linkbar; ?>
-</td>
-   <?php
-   } ?>
-<td align="right" valign="middle">
+	echo $linkbar;
+?>
+</div>
+<div class="img">
 <img src="img/htcheck.png" width="158" height="50" alt="ht://Check logo">
-</td>
-</tr>
-</table>
+</div>
+<br style="clear: both">
+
+</div>
 
 </body>
 </html>
